create table jeceira_version (
	major int,
	minor int,
	micro int,
	constraint jeceira_version_pk primary key(major, minor, micro)
);

create table node_type (
	node_type_name varchar(255),
	mixin bit default 0 not null,
	orderable_child_nodes bit default 0 not null,
	primary_item_name varchar(255),
	constraint node_type_pk primary key(node_type_name)
);

create table supertype (
	node_type_name varchar(255),
	supertype_name varchar(255),
	constraint supertype_pk primary key(node_type_name, supertype_name),
	constraint supertype_node_type foreign key(node_type_name)
		references node_type(node_type_name)
		on delete cascade,
	constraint supertype_supertype  foreign key(supertype_name)
		references node_type(node_type_name)
);

create table node_def (
	node_type_name varchar(255),
	node_def_number int,
	node_def_name varchar(255) not null,
	default_primary_type varchar(255),
	auto_created bit default 0 not null,
	mandatory bit default 0 not null,
	on_parent_version varchar(63) not null,
	protected bit default 0 not null,
	same_name_sibs bit default 0 not null,
	constraint node_def_pk primary key(node_type_name, node_def_number),
	constraint node_def_node_type foreign key(node_type_name)
		references node_type(node_type_name)
		on delete cascade,
	constraint node_def_default_primary_type_name foreign key(default_primary_type)
		references node_type(node_type_name)
);

create table property_def (
	node_type_name varchar(255),
	property_def_number int,
	property_def_name varchar(255) not null,
	required_type varchar(63) not null,
	auto_created bit default 0 not null,
	mandatory bit default 0 not null,
	on_parent_version varchar(63) not null,
	protected bit default 0 not null,
	multiple bit default 0 not null,
	constraint property_def_pk primary key(node_type_name, property_def_number),
	constraint property_def_node_type foreign key(node_type_name)
		references node_type(node_type_name)
		on delete cascade
);

create table namespace (
	prefix varchar(63),
	uri varchar(1023) not null,
	uri_usage int default 0 not null,
	constraint namespace_pk primary key(prefix),
	constraint namespace_unique_uri unique(uri),
	constraint namespace_uri_usage check(uri_usage >= 0)
);

create table node (
	node_workspace varchar(255),
    node_uuid varchar(255),
    node_name varchar(255) not null,
    node_position int not null,
    parent_workspace varchar(255),
    parent_uuid varchar(255),
	node_type_name varchar(255) not null,
	node_def_type_name varchar(255) not null,
    node_def_number int not null,
    constraint node_pk primary key(node_workspace, node_uuid),
    constraint node_parent foreign key(parent_workspace, parent_uuid)
    	references node(node_workspace, node_uuid)
    	on delete cascade,
    constraint node_primary_type foreign key(node_type_name)
    	references node_type(node_type_name)
    	on delete cascade,
    constraint node_node_def foreign key(node_def_type_name, node_def_number)
    	references node_def(node_type_name, node_def_number)
    	on delete cascade,
    constraint node_same_workspace 
    	check ((parent_workspace is null) or (node_workspace = parent_workspace))
);

create table sv_property (
	node_workspace varchar(255),
    node_uuid varchar(255),
	property_name varchar(255),
	property_type varchar(63) not null,
	binary_value varbinary,
	boolean_value bit,
	date_value timestamp,
	double_value double,
	long_value bigint,
	string_value varchar,
	property_def_type_name varchar(255) not null,
    property_def_number int not null,
    constraint sv_property_pk primary key(node_workspace, node_uuid, property_name),
    constraint sv_property_parent foreign key(node_workspace, node_uuid) 
    	references node(node_workspace, node_uuid)
    	on delete cascade,
    constraint sv_property_property_def foreign key(property_def_type_name, property_def_number)
    	references property_def(node_type_name, property_def_number)
    	on delete cascade
);

create table mv_property (
	node_workspace varchar(255),
    node_uuid varchar(255),
	property_name varchar(255),
	property_type varchar(63) not null,
	property_def_type_name varchar(255) not null,
    property_def_number int not null,
    constraint mv_property_pk primary key(node_workspace, node_uuid, property_name),
    constraint mv_property_parent foreign key(node_workspace, node_uuid) 
    	references node(node_workspace, node_uuid)
    	on delete cascade,
    constraint mv_property_property_def foreign key(property_def_type_name, property_def_number)
    	references property_def(node_type_name, property_def_number)
    	on delete cascade
);

create table mv_value (
    node_workspace varchar(255),
    node_uuid varchar(255),
    property_name varchar(255),
    value_index int,
	binary_value varbinary,
	boolean_value bit,
	date_value timestamp,
	double_value double,
	long_value bigint,
	string_value varchar,
    constraint mv_value_pk primary key(node_workspace, node_uuid, property_name, value_index),
    constraint mv_value_parent foreign key(node_workspace, node_uuid, property_name) 
    	references mv_property(node_workspace, node_uuid, property_name)
    	on delete cascade
);

create table required_primary_type (
	node_type_name varchar(255),
	node_def_number int,
	required_primary_type varchar(255),
	constraint required_primary_type_pk primary key(node_type_name, node_def_number, required_primary_type),
	constraint required_primary_type_node_def foreign key(node_type_name, node_def_number)
		references node_def(node_type_name, node_def_number)
		on delete cascade,
	constraint required_primary_type_node_type foreign key(required_primary_type)
		references node_type(node_type_name)
);

create table value_constraint (
	node_type_name varchar(255),
	property_def_number int,
	constraint_index int,
	value_constraint varchar(1023),
	constraint value_constraint_pk primary key(node_type_name, property_def_number, constraint_index),
	constraint value_constraint_property_def foreign key(node_type_name, property_def_number)
		references property_def(node_type_name, property_def_number)
		on delete cascade
);

create table default_value (
	node_type_name varchar(255),
	property_def_number int,
	value_index int,
	binary_value varbinary,
	boolean_value bit,
	date_value timestamp,
	double_value double,
	long_value bigint,
	string_value varchar,
	constraint default_value_pk primary key(node_type_name, property_def_number, value_index),
	constraint default_value_property_def foreign key(node_type_name, property_def_number)
		references property_def(node_type_name, property_def_number)
		on delete cascade
);
